using System;
using System.Collections;
using System.Configuration;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Timers;
using Microsoft.Web.Services2;
using Microsoft.Web.Services2.Addressing;
using Microsoft.Web.Services2.Messaging;
using Plumbwork.Orange.Eventing;

namespace Plumbwork.Orange.ClientCode
{
	public class WSEventingService : System.ServiceProcess.ServiceBase
	{
		private System.ComponentModel.Container components = null;
		private static string strServiceEndpointURI = ConfigurationSettings.AppSettings["ServiceEndpointURI"];
		private static int intMSMQPeriod = Convert.ToInt32(ConfigurationSettings.AppSettings["MSMQPeriod"]);
		private Timer tmrMain = new Timer();

		public WSEventingService()
		{
			// This call is required by the Windows.Forms Component Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitComponent call
		}

		// The main entry point for the process
		static void Main()
		{
			System.ServiceProcess.ServiceBase[] ServicesToRun;
	
			// More than one user Service may run within the same process. To add
			// another service to this process, change the following line to
			// create a second service object. For example,
			//
			//   ServicesToRun = new System.ServiceProcess.ServiceBase[] {new Service1(), new MySecondUserService()};
			//
			ServicesToRun = new System.ServiceProcess.ServiceBase[] { new WSEventingService() };

			System.ServiceProcess.ServiceBase.Run(ServicesToRun);
		}

		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
			this.ServiceName = "WSEventingService";
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// Start WS-Eventing service.
		/// </summary>
		protected override void OnStart(string[] args)
		{
			try
			{
				// write event log entry
				EventLog.WriteEntry("WSEventingService", "WS-Eventing service received a start request.", EventLogEntryType.Information);

				// initialize ws-eventing service
				SubscriptionService ss = new SubscriptionService();
				Uri to = new Uri(strServiceEndpointURI);
				EndpointReference EPR = new EndpointReference(to);
				
				// init soap receiver
				SoapReceivers.Add(EPR, ss);

				// init timer
				tmrMain.Interval = 1000*intMSMQPeriod;
				tmrMain.AutoReset = true;
				tmrMain.Elapsed += new ElapsedEventHandler(OnTimerEvent);

				// check if msmq queue exists
				if (MSMQ.QueueExists())
				{
					// start msmq loop
					tmrMain.Start();
				}
			}
			catch(Exception ex)
			{
				EventLog.WriteEntry("WSEventingService", "Error during initialization: " + ex.Message, EventLogEntryType.Error);
			}
		}
 
		/// <summary>
		/// Pause processing.
		/// </summary>
		protected override void OnPause()
		{
			// write event log entry
			EventLog.WriteEntry("WSEventingService", "WS-Eventing service received a pause request.", EventLogEntryType.Information);

			// stop timer
			tmrMain.Stop();
		}

		/// <summary>
		/// Continue processing.
		/// </summary>
		protected override void OnContinue()
		{
			// write event log entry
			EventLog.WriteEntry("WSEventingService", "WS-Eventing service received a resume request.", EventLogEntryType.Information);

			// start timer
			tmrMain.Start();
		}

		/// <summary>
		/// Stop everyting.
		/// </summary>
		protected override void OnStop()
		{
			// write event log entry
			EventLog.WriteEntry("WSEventingService", "WS-Eventing service received a stop request.", EventLogEntryType.Information);

			// stop timer
			tmrMain.Stop();
		}

		private static void OnTimerEvent(object source, ElapsedEventArgs e)
		{
			MSMQ.SendNotifications();
		}
	}
}
