using System;
using System.Configuration;
using System.Diagnostics;
using System.Messaging;
using System.Threading;
using Microsoft.Web.Services2;
using Microsoft.Web.Services2.Addressing;
using Microsoft.Web.Services2.Messaging;
using Plumbwork.Orange.Eventing;

namespace Plumbwork.Orange.ClientCode
{
	/// <summary>
	/// Message queue supporting functions.
	/// </summary>
	public class MSMQ
	{
		private static string strMSMQQueue = ConfigurationSettings.AppSettings["MSMQQueue"];
		private static string strNotifyAction = ConfigurationSettings.AppSettings["NotifyAction"];
		static ManualResetEvent mreSignal = new ManualResetEvent(false);
		static int intCount = 0;

		/// <summary>
		/// Checks if MSMQ queue exists, if not, it creates it.
		/// </summary>
		internal static bool QueueExists()
		{
			bool booExists = false;

			try
			{
				booExists = MessageQueue.Exists(".\\Private$\\" + strMSMQQueue);
				if (!booExists)
				{
					// create queue
					MessageQueue mq = MessageQueue.Create(".\\Private$\\" + strMSMQQueue, false);

					// set permissions
					mq.SetPermissions("Everyone", MessageQueueAccessRights.FullControl);

					// event log
					EventLog.WriteEntry("WSEventingService", "Queue did not exitst. Created queue: " + strMSMQQueue + ". Please set appropriate permissions on this queue.", EventLogEntryType.Information);

					// set exists flag
					booExists = true;
				}
			}
			catch(Exception ex)
			{
				// cannot check or create queue -> eventlog
				EventLog.WriteEntry("WSEventingService", "Cannot check if queue exists or cannot create queue. Error message: " + ex.Message, EventLogEntryType.Error);
				booExists = false;
			}
			return booExists;
		}

		/// <summary>
		/// Method sends notifications to all subscribers.
		/// </summary>
		internal static void SendNotifications()
		{
			try
			{
				MessageQueue mq = new MessageQueue(".\\Private$\\" + strMSMQQueue);
				mq.ReceiveCompleted += new ReceiveCompletedEventHandler(ReceiveCompleted);
				mq.Formatter = new XmlMessageFormatter(new Type[] {typeof(String)});
				mq.BeginReceive(new TimeSpan(0, 0, 3));
				mreSignal.WaitOne();
			}
			catch(Exception ex)
			{
				// cannot send notification
				EventLog.WriteEntry("WSEventingService", "Cannot send notification. Error message: " + ex.Message, EventLogEntryType.Error);
			}
		}

		private static void ReceiveCompleted(object source, ReceiveCompletedEventArgs asyncResult)
		{
			try
			{
				// connect to the queue
				MessageQueue mq = (MessageQueue)source;

				// end the asynchronous receive operation
				Message m = mq.EndReceive(asyncResult.AsyncResult);
				string strMessage = (string)(m.Body);
			
				// send notifications
				NotificationManager n = new NotificationManager();
				n.SendNotifications(CreateNotification(strMessage));

				// eventlog
				EventLog.WriteEntry("WSEventingService", "Notification sent. Message: " + strMessage, EventLogEntryType.Information);

				intCount += 1;
				if (intCount == 10)
				{
					mreSignal.Set();
				}

				// restart the asynchronous receive operation
				mq.BeginReceive();
			}
			catch (Exception)
			{
			}
		}

		/// <summary>
		/// Creates a SOAP envelope from MSMQ message.
		/// </summary>
		/// <returns>A soap envelope from the message in message queue.</returns>
		private static SoapEnvelope CreateNotification(string message)
		{
			SoapEnvelope envelope = new SoapEnvelope();
			try
			{
				envelope.SetBodyObject(message);
				envelope.Context.Addressing.Action = strNotifyAction;
			}
			catch(Exception ex)
			{
				// cannot send notification
				EventLog.WriteEntry("WSEventingService", "Cannot get message from message queue. Error message: " + ex.Message, EventLogEntryType.Error);
			}
			return envelope;
		}
	}
}
